package com.atlassian.confluence.test.rpc;

import com.atlassian.event.Event;

import java.util.List;
import java.util.LinkedList;
import java.util.Collections;

public class FuncTestEventQueue
{
    protected List eventList;
    public static final int MAX_EVENTS = 9;

    public FuncTestEventQueue()
    {
        this.eventList = Collections.synchronizedList(new LinkedList());
    }

    /**
     * Adds an event to the event queue.
     * If more then MAX_EVENTS events are in the queue, the oldest element will be removed.
     * @param event
     */
    public void addEvent(Event event)
    {
        if (eventList.size() > MAX_EVENTS)
        {
            eventList.remove(0);
        }
        eventList.add(event);
    }

    /**
     * returns a list of the most recent events
     * @return List of event objects.
     */
    public List getEvents()
    {
        return eventList;
    }

    /**
     * Checks whether a given event was fired and stored in the queue
     * @param event
     * @return true if the event is in the queue.
     */
    public boolean containsEvent(Event event)
    {
        return eventList.contains(event);
    }
}
